@@ -47,6 +47,7 @@ class QueryType(Enum):
     AVERAGE_IMPLIED_VOLATILITY = "Average Implied Volatility"
     AVERAGE_IMPLIED_VARIANCE = "Average Implied Variance"
     SWAP_RATE = "Swap Rate"
+    SWAP_ANNUITY = "Swap Annuity"
     BASIS_SWAP_RATE = "Basis Swap Rate"
     SWAPTION_VOL = "Swaption Vol"
     MIDCURVE_VOL = "Midcurve Vol"
@@ -67,6 +68,7 @@ class QueryType(Enum):
     POLICY_RATE_EXPECTATION = "Policy Rate Expectation"
     CENTRAL_BANK_SWAP_RATE = "Central Bank Swap Rate"
     FORWARD_PRICE = "Forward Price"
+    FAIR_PRICE = "Fair Price"
     SPOT = "Spot"
     ES_NUMERIC_SCORE = "ES Numeric Score"
     ES_NUMERIC_PERCENTILE = "ES Numeric Percentile"
@@ -106,7 +108,7 @@ class GsDataApi(DataApi):
             xref_keys = set(where.keys()).intersection(XRef.properties())
             if xref_keys:
                 # Check that assetId is a symbol dimension of this data set. If not, we need to do a separate query
-                # to resolve xref --> assetId
+                # to resolve xref pip install dtaidistance--> assetId
                 if len(xref_keys) > 1:
                     raise MqValueError('Cannot not specify more than one type of asset identifier')
 
@@ -247,7 +249,7 @@ class GsDataApi(DataApi):
         return res
 
     @staticmethod
-    def build_market_data_query(asset_ids: List[str], query_type: QueryType, where: Union[FieldFilterMap] = None,
+    def build_market_data_query(asset_ids: List[str], query_type: QueryType, where: Union[FieldFilterMap, dict] = None,
                                 source: Union[str] = None, real_time: bool = False):
         inner = {
             'assetIds': asset_ids,
