@@ -14,10 +14,12 @@ specific language governing permissions and limitations
 under the License.
 """
 from abc import ABCMeta, abstractmethod
+import datetime as dt
 import logging
 from typing import Iterable, Mapping, Optional, Tuple, Union
 
-from gs_quant.base import PricingKey
+from gs_quant.base import RiskKey
+from gs_quant.markets import ClosingMarket, LiveMarket
 from gs_quant.risk import ErrorValue, RiskRequest
 from gs_quant.risk.result_handlers import result_handlers
 
@@ -46,30 +48,30 @@ class RiskApi(metaclass=ABCMeta):
     def _handle_results(cls, request: RiskRequest, results: Iterable) -> dict:
         formatted_results = {}
 
-        pricing_key = PricingKey(
-            request.pricing_and_market_data_as_of,
-            request.pricing_location.value,
-            request.parameters,
-            request.scenario
-        )
+        for risk_measure, position_results in zip(request.measures, results):
+            for position, date_results in zip(request.positions, position_results):
+                for as_of, date_result in zip(request.pricing_and_market_data_as_of, date_results):
+                    handler = result_handlers.get(date_result.get('$type'))
 
-        for measure_idx, position_results in enumerate(results):
-            risk_measure = request.measures[measure_idx]
+                    # TODO Handle this better
+                    market = LiveMarket(request.pricing_location) if isinstance(as_of.market_data_as_of, dt.datetime)\
+                        else ClosingMarket(request.pricing_location, as_of.market_data_as_of)
 
-            for position_idx, date_results in enumerate(position_results):
-                if len(date_results) != len(pricing_key):
-                    raise RuntimeError('Number of results did not match requested days')
+                    risk_key = RiskKey(
+                        cls,
+                        as_of.pricing_date,
+                        market,
+                        request.parameters,
+                        request.scenario,
+                        risk_measure
+                    )
 
-                handler = result_handlers.get(date_results[0].get('$type'))
-                position = request.positions[position_idx]
+                    try:
+                        result = handler(date_result, risk_key, position.instrument) if handler else date_result
+                    except Exception as e:
+                        result = ErrorValue(risk_key, str(e))
+                        _logger.error(result)
 
-                try:
-                    date_results = handler(date_results, pricing_key, position.instrument) if handler else date_results
-                except Exception as e:
-                    error = str(e)
-                    date_results = ErrorValue(pricing_key, error)
-                    _logger.error(error)
-
-                formatted_results.setdefault(risk_measure, {})[position] = date_results
+                    formatted_results[(risk_key, position.instrument)] = result
 
         return formatted_results
