@@ -98,11 +98,11 @@ def test_lag():
     dates = pd.date_range("2019-01-01", periods=4, freq="D")
     x = pd.Series([1.0, 2.0, 3.0, 4.0], index=dates)
 
-    result = lag(x)
+    result = lag(x, mode=LagMode.TRUNCATE)
     expected = pd.Series([np.nan, 1.0, 2.0, 3.0], index=dates)
     assert_series_equal(result, expected, obj="Lag")
 
-    result = lag(x, 2)
+    result = lag(x, 2, LagMode.TRUNCATE)
     expected = pd.Series([np.nan, np.nan, 1.0, 2.0], index=dates)
     assert_series_equal(result, expected, obj="Lag 2")
 
@@ -110,6 +110,10 @@ def test_lag():
     expected = pd.Series([np.nan, np.nan, 1.0, 2.0, 3.0, 4.0], index=pd.date_range("2019-01-01", periods=6, freq="D"))
     assert_series_equal(result, expected, obj="Lag 2 Extend")
 
+    result = lag(x, 2)
+    expected = pd.Series([np.nan, np.nan, 1.0, 2.0, 3.0, 4.0], index=pd.date_range("2019-01-01", periods=6, freq="D"))
+    assert_series_equal(result, expected, obj="Lag 2 Extend")
+
     y = pd.Series([0] * 4, index=pd.date_range('2020-01-01T00:00:00Z', periods=4, freq='S'))
     with pytest.raises(Exception):
         lag(y, 5, LagMode.EXTEND)
